@@ -18,11 +18,11 @@ $redis = AnyEvent::Redis::RipeRedis->new(
   encoding => 'utf8',
 
   on_connect => sub {
-    say 'Connected to Redis';
+    say 'Connected to Redis server';
   },
 
   on_disconnect => sub {
-    say 'Disconnected from Redis';
+    say 'Disconnected from Redis server';
   },
 
   on_error => sub {
@@ -61,3 +61,5 @@ my $term_watcher = AnyEvent->signal(
 );
 
 $cv->recv();
+
+$redis->disconnect();
